import argparse
import os
import sys
from configparser import ConfigParser, NoOptionError

from .core import Runner
from .gettext import _

LICENSE = """
The MIT License (MIT)
Copyright (c) 2013−2014, 2022 OEC Project

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is furnished
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""


def main(arguments=None) -> int:
    config = ConfigParser()

    parser = argparse.ArgumentParser(
        description=_("Load chunks of data into Odoo"),
    )
    parser.add_argument(
        "-i",
        "--infile",
        metavar="filename",
        type=str,
        default="STDIN",
        help=_("input file to read for data (default: STDIN)"),
    )
    parser.add_argument(
        "-c",
        "--config",
        metavar="configfile",
        type=str,
        default=None,
        help=_("config file for oec"),
    )
    parser.add_argument(
        "-s",
        "--server",
        metavar="servername",
        type=str,
        default=None,
        help=_("Server name or IP address"),
    )
    parser.add_argument(
        "-p",
        "--port",
        metavar="port",
        type=str,
        default=None,
        help=_("Port of the Odoo server"),
    )
    parser.add_argument(
        "-P",
        "--protocol",
        metavar="protocol",
        type=str,
        default=None,
        help=_("protocol of your server"),
        choices=("http", "https"),
    )
    parser.add_argument(
        "-u",
        "--user",
        metavar="username",
        type=str,
        default=None,
        help=_("Username to use for Odoo login"),
    )
    parser.add_argument(
        "-x",
        "--password",
        metavar="password",
        type=str,
        default=None,
        help=_("Password to use for logon"),
    )
    parser.add_argument(
        "-d",
        "--database",
        metavar="database",
        type=str,
        default=None,
        help=_("Odoo database"),
    )
    parser.add_argument(
        "-t",
        "--type",
        metavar="model",
        type=str,
        required=False,
        help=_("Model to send (ex: crm.lead)"),
    )
    parser.add_argument(
        "-m",
        "--mode",
        metavar="mode",
        type=str,
        default="auto",
        help=_("unlink, create, write or auto (default: auto)"),
    )
    parser.add_argument("-l", "--license", action="store_true")

    args = parser.parse_args(arguments)

    if args.license:
        print(LICENSE)

        return 0

    if args.config and not os.path.exists(args.config):
        print(_("Config file does not exist: {}").format(args.config))
        return 1

    if args.config:
        config.read(args.config)

    oeprotocol = None
    if args.config:
        try:
            oeprotocol = config.get("server", "protocol")
        except NoOptionError:
            oeprotocol = "http"

    if args.protocol:
        # command line always overrides config file
        oeprotocol = args.protocol

    oeserver = None
    if args.config:
        try:
            oeserver = config.get("server", "name")
        except NoOptionError:
            pass
    if args.server:
        # command line always overrides config file
        oeserver = args.server

    missing_required_info = False

    if not oeserver:
        print(_("Server is required either in config or command line"))
        missing_required_info = True

    oeport = None
    if args.config:
        try:
            oeport = config.get("server", "port")
        except NoOptionError:
            pass
    if args.port:
        # command line always overrides config file
        oeport = args.port

    if not oeport:
        print(_("Port is required either in config or command line"))
        missing_required_info = True

    oeuser = None
    if args.config:
        try:
            oeuser = config.get("server", "user")
        except NoOptionError:
            pass
    if args.user:
        # command line always overrides config file
        oeuser = args.user

    if not oeuser:
        print(_("User is required either in config or command line"))
        missing_required_info = True

    oepass = None
    if args.config:
        try:
            oepass = config.get("server", "password")
        except NoOptionError:
            pass
    if args.password:
        # command line always overrides config file
        oepass = args.password

    if not oepass:
        print(_("Password is required either in config or command line"))
        missing_required_info = True

    oedb = None
    if args.config:
        try:
            oedb = config.get("server", "database")
        except NoOptionError:
            pass
    if args.database:
        # command line always overrides config file
        oedb = args.database

    if not oedb:
        print(_("Database is required either in config or command line"))
        missing_required_info = True

    if missing_required_info:
        return 1

    runner = Runner(
        args.infile,
        args.type,
        args.mode,
        oeprotocol,
        oeserver,
        oeport,
        oeuser,
        oepass,
        oedb,
    )
    all_success = runner.run()
    if not all_success:
        return 2
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
