import datetime
import sys

from .builder import Builder
from .gettext import _
from .reader import Reader
from .resultparser import ResultParser
from .sender import Sender


class Runner:
    """this is the code implementation of OEC
    It will be instanciated by a cmd line or any other
    means in the future.
    It will use the provided arguments to read a given
    source and send the data to a Builder
    in charge of creating the corresponding payloads
    and then send the data to Odoo using a Sender instance
    """

    def __init__(
        self,
        infilename,
        itemtype,
        mode,
        oeprotocol,
        oeserver,
        oeport,
        oeuser,
        oepass,
        oedb,
    ):

        # first open the in_filename
        self.source = self.__open_source__(infilename)

        # store other args
        self.itemtype = itemtype
        self.mode = mode
        self.oeprotocol = oeprotocol
        self.oeserver = oeserver
        self.oeport = oeport
        self.oeuser = oeuser
        self.oepass = oepass
        self.oedb = oedb

    def run(self) -> bool:
        """public method to run our import system"""

        start_time = datetime.datetime.now()

        # iterate over source with a reader
        reader = Reader(self.source)

        # then create instrumentize a builder to create ready to send
        # data flow
        builder = Builder(reader, self.itemtype)

        # Sender is then responsible to use this data flow and send however
        # it wants the data to Odoo
        sender = Sender(
            self.mode,
            self.oeprotocol,
            self.oeserver,
            self.oeport,
            self.oeuser,
            self.oepass,
            self.oedb,
            builder,
        )
        result = sender.send()

        result_parser = ResultParser(result)
        index = 0
        all_success = True
        for index, item in enumerate(result_parser.parse_result()):
            success, info = item
            if not success:
                all_success = False
                message, sent_item = info
                sys.stderr.write(_("Odoo Error message:"))
                sys.stderr.write(f"\n{message}\n")
                sys.stderr.write(_("Line number: {}").format(index + 1))
                sys.stderr.write(f"\n{sent_item}\n")

        end_time = datetime.datetime.now()
        duration = end_time - start_time
        print(_("Import done in {}").format(duration))
        if index != 0:
            print(_("Average time per item: {}").format(duration / index))

        return all_success

    def __open_source__(self, in_filename):
        """internal helper that will either open a real filename
        or interpret the STDIN filename as sys.stdin
        """
        if in_filename == "STDIN":
            f = sys.stdin

        else:
            f = open(in_filename, "rb")

        return f
